package com.gree.mall.manager.logic.admin;

import cn.hutool.json.JSONObject;
import com.aliyuncs.utils.StringUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import com.google.code.kaptcha.impl.DefaultKaptcha;
import com.gree.mall.manager.bean.ExcelData;
import com.gree.mall.manager.bean.SVerification;
import com.gree.mall.manager.bean.admin.AdminModuleTree;
import com.gree.mall.manager.bean.admin.AdminUserBean;
import com.gree.mall.manager.bean.admin.AdminUserCom;
import com.gree.mall.manager.bean.admin.ExternalMapBean;
import com.gree.mall.manager.bean.admin.reqDto.AdminCompanyWechatReqBean;
import com.gree.mall.manager.bean.admin.reqDto.AdminUserAddReqBean;
import com.gree.mall.manager.commonmapper.AdminMapper;
import com.gree.mall.manager.enums.RedisPrefixEnum;
import com.gree.mall.manager.enums.RoleTypeEnum;
import com.gree.mall.manager.exception.RemoteServiceException;
import com.gree.mall.manager.logic.common.CommonLogic;
import com.gree.mall.manager.plus.entity.*;
import com.gree.mall.manager.plus.service.*;
import com.gree.mall.manager.utils.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class AdminUserLogic {
    @Autowired
    RedisUtil redisUtil;
    @Autowired
    DefaultKaptcha defaultKaptcha;
    @Autowired
    CommonLogic commonLogic;
    @Autowired
    AdminUserService adminUserService;
    @Autowired
    AdminModuleService adminModuleService;
    @Autowired
    AdminRoleService adminRoleService;
    @Autowired
    AdminUserModuleRelaService adminUserModuleRelaService;
    @Autowired
    AdminUserCompanyRelaService adminUserCompanyRelaService;
    @Autowired
    AdminCompanyWechatService adminCompanyWechatService;
    @Autowired
    AdminWebsitService adminWebsitService;
    @Autowired
    AdminUserWebsitRelaService adminUserWebsitRelaService;
    @Autowired
    AdminMapper adminMapper;
    @Value("${front.top.url}")
    private String frontTopUrl;

    /**
     * 验证码
     */
    public SVerification defaultKaptcha() throws IOException {
        // 生成文字验证码
        String text = defaultKaptcha.createText();

        // 生成图片验证码
        ByteArrayOutputStream outputStream = null;
        BufferedImage image = defaultKaptcha.createImage(text);
        outputStream = new ByteArrayOutputStream();
        ImageIO.write(image, "png", outputStream);
        byte b[] = outputStream.toByteArray();//从流中获取数据数组。
//        String baseStr = new BASE64Encoder().encode(b);
        String baseStr = Base64.getEncoder().encodeToString(b);
        //生成一个token
        String code = UUID.randomUUID().toString();
        redisUtil.set(RedisPrefixEnum.TOKEN_VERIFICATION + code, text, 2 * 60);
        SVerification verification = new SVerification();
        verification.setCode(code);
        verification.setPic(baseStr);
        return verification;
    }

    /**
     * 登录
     */
    public AdminUserBean login(String userName, String password, String code, String codeValue) throws RemoteServiceException {
        String value = (String) redisUtil.get(RedisPrefixEnum.TOKEN_VERIFICATION + code);
        if (!value.equals(codeValue)) {
            throw new RemoteServiceException("验证码错误");
        }
        AdminUser adminUser = adminUserService.lambdaQuery().eq(AdminUser::getUserName, userName).eq(AdminUser::getPassword, MD5Utils.md5(password)).one();
        if (adminUser == null) {
            throw new RemoteServiceException("帐号密码错误");
        }
        if (!adminUser.getStatus()) {
            throw new RemoteServiceException("帐号冻结中，请联系相关管理人员");
        }
        adminUser.setLastLoginTime(new Date());
        adminUser.updateById();

        AdminUserBean adminUserBean = new AdminUserBean();
        BeanUtils.copyProperties(adminUser, adminUserBean);
        //不返回密码
        adminUserBean.setPassword(null);
        adminUserBean.setToken(JwtUtils.createJWT(adminUser.getAdminUserId(), adminUser.getNickName(), adminUser.getUserName(), 20 * 60 * 60 * 1000));
        //查询拥有的权限
        List<AdminUserCompanyRela> list = adminUserCompanyRelaService.lambdaQuery().eq(AdminUserCompanyRela::getAdminUserId, adminUser.getAdminUserId()).list();
        adminUserBean.setAdminUserCompanyRelaList(list);
        //存放到redis
        redisUtil.set(RedisPrefixEnum.TOKEN_MGR.toString() + adminUserBean.getToken(), adminUser, 20 * 60 * 60 * 1000);
        return adminUserBean;
    }

    /**
     * 修改密码
     */
    public void updatePassword(String userName, String password, String newPwd) throws RemoteServiceException {
        AdminUser adminUser = adminUserService.lambdaQuery().eq(AdminUser::getUserName, userName)
                .eq(AdminUser::getPassword, MD5Utils.md5(password)).one();
        if (adminUser == null) {
            throw new RemoteServiceException("帐号密码错误");
        }
        adminUser.setPassword(MD5Utils.md5(newPwd));
        adminUserService.updateById(adminUser);
    }

    /**
     * 修改帐号状态
     */
    public void updateStatus(String adminUserId, Boolean status) {
        adminUserService.lambdaUpdate().set(AdminUser::getStatus, status).eq(AdminUser::getAdminUserId, adminUserId).update();
    }

    /**
     * 重置密码
     */
    public void resetPassword(String adminUserId, String password, HttpServletRequest request) throws RemoteServiceException {
        //检查是否拥有重置的密码权限
        String nickName = CommonUtils.getNickName(request);
        if (!nickName.equals("admin")) {
            throw new RemoteServiceException("只有超级管理员才有重置密码的权限");
        }

        //开始重置
        AdminUser adminUser = adminUserService.getById(adminUserId);
        adminUser.setPassword(MD5Utils.md5(password));
        adminUser.updateById();
    }


    /**
     * 新增帐号
     */
    @Transactional
    public void add(AdminUserAddReqBean adminUser, HttpServletRequest request) throws RemoteServiceException {

        AdminUserCom admin = commonLogic.getAdminUser(request);

        if (StringUtils.isEmpty(adminUser.getUserName()) || StringUtils.isEmpty(adminUser.getPassword())) {
            throw new RemoteServiceException("参数错误");
        }
        Integer count = adminUserService.lambdaQuery().eq(AdminUser::getUserName, adminUser.getUserName()).count();
        if (count > 0) {
            throw new RemoteServiceException("帐号已被注册");
        }
        //需要用户roleId
        AdminRole loginRole = adminRoleService.getById(admin.getRoleId());
        AdminRole addUserRole = null;
        if (StringUtils.isEmpty(adminUser.getRoleId()) ||
                (addUserRole = adminRoleService.getById(adminUser.getRoleId())) == null) {
            throw new RemoteServiceException("请为用户选择角色");
        }
        adminUser.setRoleName(addUserRole.getName());
        //  添加用户的角色级别 <= 被添加的用户的角色级别
        if (loginRole.getType() <= addUserRole.getType()) {
            throw new RemoteServiceException("无法为用户赋予与自身级别相当的角色");
        }
        //用户级别默认取角色级别
        if (adminUser.getType() == null) {
            //数据库和角色表 type值不一致
            adminUser.setType(addUserRole.getType() - 1);
        }
        if (adminUser.getType() != (addUserRole.getType() - 1)) {
            throw new RemoteServiceException("所选角色级别和用户级别不一致!!");
        }


        //部门账号
        if (adminUser.getType() == 0) {
            //默认登录用户的CompanyWechatId (admin则无)
            adminUser.setCompanyWechatId(admin.getCompanyWechatId());
            adminUser.setCompanyName(admin.getCompanyName());
        }
        adminUser.setPassword(MD5Utils.md5(adminUser.getPassword()));
        adminUser.setCreateTime(new Date());
        adminUserService.save(adminUser);

        //部门账号
        if (adminUser.getType() == 0) {
            //添加部门
            this.addAdminUserWebsit(adminUser);
            //todo admin 添加的部门没有companyWechatId
        }
        //公司主账号
        else if (adminUser.getType() == 1) {
            AdminCompanyWechat companyWechat = null;
            if (StringUtils.isEmpty(adminUser.getCompanyWechatId()) || (companyWechat = adminCompanyWechatService.getById(adminUser.getCompanyWechatId())) == null) {
                throw new RemoteServiceException("admin添加公司主账号时候,必须要有companyWechatId");
            }
            AdminUserCompanyRela adminUserCompanyRela = new AdminUserCompanyRela();
            adminUserCompanyRela.setAdminUserId(adminUser.getAdminUserId());
            adminUserCompanyRela.setCompanyWechatId(companyWechat.getCompanyWechatId());
            adminUserCompanyRela.setCompanyName(companyWechat.getCompanyName());
            adminUserCompanyRela.setCreateTime(new Date());
            adminUserCompanyRela.insert();
        }

    }

    private void signUpCompany(AdminUserBean adminUser, String createBy) {
        //查询企业负责人的角色帐号
        AdminRole adminRole = adminRoleService.lambdaQuery().eq(AdminRole::getName, "企业负责人").one();
        if (adminRole != null) {
            adminUser.setRoleId(adminRole.getAdminRoleId());
            adminUser.setRoleName(adminRole.getName());
            adminUser.updateById();
        }

        int count1 = adminCompanyWechatService.count();
        AdminCompanyWechat adminCompanyWechat = new AdminCompanyWechat();
        BeanUtils.copyProperties(adminUser, adminCompanyWechat);
        adminCompanyWechat.setCompanyName(adminUser.getNickName());
        adminCompanyWechat.setAgentId2((count1 + 1) * 2);
        adminCompanyWechat.setAgentId3((count1 + 1) * 3);
        adminCompanyWechat.setCreateTime(new Date());
        adminCompanyWechat.setCreateBy(createBy);
        adminCompanyWechat.insert();


        adminUser.setCompanyWechatId(adminCompanyWechat.getCompanyWechatId());
        adminUser.setCompanyName(adminCompanyWechat.getCompanyName());
        adminUser.updateById();

//        AdminWebsit adminWebsit = adminWebsitService.lambdaQuery()
//                .eq(AdminWebsit::getCorpId, adminCompanyWechat.getCorpId())
//                .eq(AdminWebsit::getWebsitId, 1).one();
//
//        AdminUserWebsitRela adminUserWebsitRela = new AdminUserWebsitRela();
//        adminUserWebsitRela.setAdminUserId(adminUser.getAdminUserId());
//        adminUserWebsitRela.setAdminWebsitId(adminWebsit.getId());
//        adminUserWebsitRela.setCreateTime(new Date());
//        adminUserWebsitRela.insert();
        //刷新配置
        commonLogic.reloadWxConfig();
    }

    //添加用户部门关系
    private void addAdminUserWebsit(AdminUserAddReqBean adminUser) {
        //先清除
        if (CollectionUtils.isEmpty(adminUser.getAdminWebsitIds())) {
            return;
        }
        adminUserWebsitRelaService.lambdaUpdate().eq(AdminUserWebsitRela::getAdminUserId, adminUser.getAdminUserId()).remove();
        List<AdminWebsit> websits = adminWebsitService.lambdaQuery().in(AdminWebsit::getId, adminUser.getAdminWebsitIds()).list();
        if (websits.size() == 0) {
            throw new RemoteServiceException("请选择至少1个部门");
        }
        log.info("【添加用户部门关系】:{}", com.alibaba.fastjson.JSONObject.toJSONString(adminUser));
        for (AdminWebsit adminWebsit : websits) {
            AdminUserWebsitRela adminUserWebsitRela = new AdminUserWebsitRela();
            adminUserWebsitRela.setAdminUserId(adminUser.getAdminUserId());
            adminUserWebsitRela.setAdminWebsitId(adminWebsit.getId());
            adminUserWebsitRela.setCompanyWechatId(adminUser.getCompanyWechatId());
            adminUserWebsitRela.setCreateTime(new Date());
            adminUserWebsitRela.insert();
        }

    }

    /**
     * 修改帐号
     */
    public void update(HttpServletRequest request, AdminUserAddReqBean newUser) {

        AdminUserCom admin = commonLogic.getAdminUser(request);

        AdminUser oldUser = null;
        if (StringUtils.isEmpty(newUser.getAdminUserId()) ||
                (oldUser = adminUserService.getById(newUser.getAdminUserId())) == null) {
            throw new RemoteServiceException("请选择用户");
        }

        //需要用户roleId
        AdminRole loginRole = adminRoleService.getById(admin.getRoleId());
        AdminRole addUserRole = null;
        if (StringUtils.isEmpty(newUser.getRoleId()) || (addUserRole = adminRoleService.getById(newUser.getRoleId())) == null) {
            throw new RemoteServiceException("请为用户选择角色");
        }
        //  添加用户的角色级别 <= 被添加的用户的角色级别
        if (loginRole.getType() <= addUserRole.getType()) {
            throw new RemoteServiceException("无法为用户赋予与自身级别相当的角色");
        }

        //部门账号
        if (newUser.getType() == 0) {
            //默认登录用户的CompanyWechatId (admin则无)
            newUser.setCompanyWechatId(admin.getCompanyWechatId());
            newUser.setCompanyName(admin.getCompanyName());
        }
        newUser.setUserName(null);
        newUser.setPassword(StringUtils.isEmpty(newUser.getPassword()) ? null : MD5Utils.md5(newUser.getPassword()));

        newUser.updateById();

        if (newUser.getType() != (addUserRole.getType() - 1)) {
            throw new RemoteServiceException("所选角色级别和用户级别不一致!!");
        }

        //当前为公司主账号
        if (newUser.getType() == 1) {
            //删除部门关系
            adminUserWebsitRelaService.lambdaUpdate().eq(AdminUserWebsitRela::getAdminUserId, newUser.getAdminUserId()).remove();
            //删除企业关系
            adminUserCompanyRelaService.lambdaUpdate().eq(AdminUserCompanyRela::getAdminUserId, newUser.getAdminUserId()).remove();
            //新增用户企业关系
            AdminUserCompanyRela adminUserCompanyRela = new AdminUserCompanyRela();
            adminUserCompanyRela.setAdminUserId(newUser.getAdminUserId());
            adminUserCompanyRela.setCompanyWechatId(newUser.getCompanyWechatId());
            adminUserCompanyRela.setCompanyName(newUser.getCompanyName());
            adminUserCompanyRela.setCreateTime(new Date());
            adminUserCompanyRela.insert();
        }

        //当前为为部门账号
        if (newUser.getType() == 0) {
            //添加部门
            this.addAdminUserWebsit(newUser);
            //删除企业关系
            adminUserCompanyRelaService.lambdaUpdate().eq(AdminUserCompanyRela::getAdminUserId, newUser.getAdminUserId()).remove();
            //todo admin添加的部门没有companyWechatId
        }

    }


    /**
     * 运营后台用户列表
     *
     * @return
     */
    public IPage<AdminUser> list(String roleId, Boolean status, String userName, Integer pageNum, Integer pageSize, HttpServletRequest request) {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        IPage<AdminUser> page = adminUserService.lambdaQuery()
                .eq(StringUtils.isNotEmpty(roleId), AdminUser::getRoleId, roleId)
                .eq(status != null, AdminUser::getStatus, status)
                .in(CollectionUtils.isNotEmpty(adminUser.getCompanyWechatIds()), AdminUser::getCompanyWechatId, adminUser.getCompanyWechatIds())
                .like(StringUtils.isNotEmpty(userName), AdminUser::getUserName, userName).page(new Page<>(pageNum, pageSize));
        return page;
    }


    /**
     * 商户列表
     */
    public IPage<AdminUser> list(String userName, String nickName, String linkPhone, String email, Boolean status, Integer pageNum, Integer pageSize) {
        IPage<AdminUser> adminUserIPage = adminMapper.mchList(new Page(pageNum, pageSize), userName, nickName, linkPhone, email, status);
        return adminUserIPage;
    }


    /**
     * 导出商户列表
     */
    public ExcelData exportMch(List<AdminUser> list) {
        String[] titles = new String[]{"用户名", "负责人", "联系电话", "邮箱", "地址", "创建时间"};
        List<List<Object>> rows = new ArrayList<>();
        for (AdminUser adminUser : list) {
            List<Object> row = new ArrayList<>();
            row.add(adminUser.getUserName());
            row.add(adminUser.getNickName());
            row.add(adminUser.getLinkPhone());
            row.add(adminUser.getEmail());
            row.add(adminUser.getAddress());
            row.add(DateUtils.formatDate(adminUser.getCreateTime()));
            rows.add(row);
        }
        ExcelData excelData = new ExcelData();
        excelData.setRows(rows);
        excelData.setTitles(Arrays.asList(titles));
        excelData.setName("sheet1");
        return excelData;
    }

    /**
     * 运营后台用户详情
     *
     * @param adminUserId
     * @return
     */
    public AdminUserBean detail(String adminUserId) {
        AdminUser adminUser = adminUserService.getById(adminUserId);
        AdminUserBean adminUserBean = new AdminUserBean();
        BeanUtils.copyProperties(adminUser, adminUserBean);

        List<AdminUserCompanyRela> list = adminUserCompanyRelaService.lambdaQuery().eq(AdminUserCompanyRela::getAdminUserId, adminUser.getAdminUserId()).list();
        if (list.size() > 0) {
            String companyWechatId = list.get(0).getCompanyWechatId();
            AdminCompanyWechat adminCompanyWechat = adminCompanyWechatService.getById(companyWechatId);
            if (org.apache.commons.lang3.StringUtils.isNotBlank(adminCompanyWechat.getRefundPassword())) {
                adminUserBean.setIsRefundPassword(Boolean.TRUE);
            }

            BeanUtils.copyProperties(adminCompanyWechat, adminUserBean);
            return adminUserBean;
        }
        List<AdminUserWebsitRela> websitIds = adminUserWebsitRelaService.lambdaQuery().eq(AdminUserWebsitRela::getAdminUserId, adminUser.getAdminUserId()).list();
        if (websitIds.size() > 0) {
            List<String> collect = websitIds.stream().map(AdminUserWebsitRela::getAdminWebsitId).collect(Collectors.toList());
            adminUserBean.setAdminWebsitIds(collect);
        }

        return adminUserBean;
    }


    /**
     * 退出登录
     */
    public void logout(HttpServletRequest request) {
        String token = CommonUtils.getToken(request);
        redisUtil.del(RedisPrefixEnum.TOKEN_MGR + token);
    }


    /**
     * 根据登录帐号查询权限模块
     */
    public List<AdminModuleTree> queryAdminModule(String adminUserId) {
        //String adminUserId = CommonUtils.getUserId(request);
        AdminUser adminUser = adminUserService.getById(adminUserId);
        if (adminUser.getUserName().equals("admin")) {
            return this.queryAdminModuleTree(null, true, null, null);
        } else {
            return this.queryAdminModuleTree(adminUser.getRoleId(), false, null, null);
        }
    }

    /**
     * 给角色授权
     */
    @Transactional
    public void grantModules(String roleId, List<String> adminModuleIds) {
        //1.清空该角色的权限
        adminUserModuleRelaService.lambdaUpdate().eq(AdminUserModuleRela::getAdminRoleId, roleId).remove();

        //2.生成权限
        List<AdminUserModuleRela> list = new ArrayList<>();
        for (String adminModuleId : adminModuleIds) {
            AdminUserModuleRela adminUserModuleRela = new AdminUserModuleRela();
            adminUserModuleRela.setAdminRoleId(roleId);
            adminUserModuleRela.setCreateTime(new Date());
            adminUserModuleRela.setStatus(true);
            adminUserModuleRela.setAdminModuleId(adminModuleId);
            list.add(adminUserModuleRela);
        }
        adminUserModuleRelaService.saveBatch(list);
    }

    /**
     * 根据角色id查询拥有的权限ids
     */
    public List<String> queryModuleIdChecked(String adminRoleId) {
        AdminRole adminRole = adminRoleService.getById(adminRoleId);
        Boolean admin = adminRole.getName().equals("超级管理员");
        if (admin) {
            List<AdminModule> list = adminModuleService.list();
            if (list.size() == 0) {
                return new ArrayList<>();
            }
            return list.stream().map(AdminModule::getModuleId).collect(Collectors.toList());
        }
        List<AdminUserModuleRela> list = adminUserModuleRelaService.lambdaQuery()
                .eq(AdminUserModuleRela::getAdminRoleId, adminRoleId).list();
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list.stream().map(AdminUserModuleRela::getAdminModuleId).collect(Collectors.toList());
    }

    /**
     * 根据角色查询所有功能模块权限
     */
    public List<AdminModuleTree> queryAllAdminModuleTree(String adminRoleId, HttpServletRequest request) {
        String userId = CommonUtils.getUserId(request);
        AdminUser adminUser = adminUserService.getById(userId);

        //更改的角色
        AdminRole adminRole = adminRoleService.getById(adminRoleId);
        //查询该角色有的权限d
        List<AdminUserModuleRela> hasModules = adminUserModuleRelaService.lambdaQuery().eq(AdminUserModuleRela::getAdminRoleId, adminRoleId).list();
        List<String> collect = new ArrayList<>();
        if (!CollectionUtils.isEmpty(hasModules)) {
            collect = hasModules.stream().map(AdminUserModuleRela::getAdminModuleId).collect(Collectors.toList());
        }
        List<AdminModuleTree> adminModuleTrees = this.queryAdminModuleTree(adminRoleId, adminRole.getName().equals("超级管理员"), collect, adminUser);
        return adminModuleTrees;
    }


    private List<AdminModuleTree> queryAdminModuleTree(String roleId, Boolean admin, List<String> hasModule, AdminUser adminUser) {

        List<AdminModule> adminModules = null;

        if (admin != null && !admin) {
            //非超管角色
            List<AdminUserModuleRela> list = adminUserModuleRelaService.lambdaQuery()
                    .eq(StringUtils.isNotEmpty(roleId), AdminUserModuleRela::getAdminRoleId, roleId)
                    .list();
            if (list.size() == 0) {
                return new ArrayList<>();
            }
            List<String> moduleIds = list.stream().map(AdminUserModuleRela::getAdminModuleId).collect(Collectors.toList());
            adminModules = adminModuleService.lambdaQuery().in(AdminModule::getModuleId, moduleIds)
                    .orderByAsc(AdminModule::getSortNum).list();
        } else {
            //超管角色
            List<AdminUserModuleRela> list = adminUserModuleRelaService.lambdaQuery().list();
            if (list.size() == 0) {
                return new ArrayList<>();
            }
            adminModules = adminModuleService.lambdaQuery().orderByAsc(AdminModule::getSortNum).list();
        }
        List<AdminModuleTree> trees = new ArrayList<>();
        for (AdminModule adminModule : adminModules) {
            AdminModuleTree adminModuleTree = new AdminModuleTree();
            BeanUtils.copyProperties(adminModule, adminModuleTree);
            if ((hasModule != null && hasModule.contains(adminModule.getModuleId())) || admin) {
                adminModuleTree.setShow(true);
            }
            trees.add(adminModuleTree);
        }
        List<AdminModuleTree> parents = trees.stream().filter(v -> v.getLevel() == 1).collect(Collectors.toList());
        for (AdminModuleTree adminModuleTree : parents) {
            adminModuleTree.setChildren(this.treeModule(trees, adminModuleTree.getModuleId(), hasModule, admin));
        }
        return parents;
    }

    private List<AdminModuleTree> treeModule(List<AdminModuleTree> list, String parentId, List<String> hasModule, Boolean admin) {
        //最父级资源树
        if (StringUtils.isEmpty(parentId)) {
            return new ArrayList<>();
        }
        List<AdminModuleTree> collect = list.stream().filter(e -> e.getParentId().equals(parentId)).collect(Collectors.toList());
        for (AdminModuleTree bean : collect) {
            if ((hasModule != null && hasModule.contains(bean.getModuleId())) || admin) {
                bean.setShow(true);
            }
            bean.setChildren(this.treeModule(list, bean.getModuleId(), hasModule, admin));
        }
        return collect;
    }


    public ExternalMapBean checkOrLoginAccount(String account, String password, Boolean isCheck, String timestamp, String sign) {
        String newSign = JiaSMUtil.jiasmDigest(account + password, timestamp);
        if (!newSign.equals(sign)) {
            throw new RemoteServiceException("请求不合法");
        }
        AdminUser adminUser = adminUserService.lambdaQuery()
                .eq(AdminUser::getUserName, account)
                .eq(AdminUser::getPassword, MD5Utils.md5(password))
                .one();
        if (Objects.isNull(adminUser)) {
            throw new RemoteServiceException(4040001, "请求不合法");
        }

        if (!adminUser.getStatus()) {
            throw new RemoteServiceException(4040001, "帐号冻结中，请联系相关管理人员");
        }
        if (!isCheck) {
            adminUser.setLastLoginTime(new Date());
            adminUser.updateById();

            AdminUserBean adminUserBean = new AdminUserBean();
            BeanUtils.copyProperties(adminUser, adminUserBean);
            //不返回密码
            adminUserBean.setPassword(null);
            adminUserBean.setToken(JwtUtils.createJWT(adminUser.getAdminUserId(), adminUser.getNickName(), adminUser.getUserName(), 20 * 60 * 60 * 1000));

            //存放到redis
            redisUtil.set(RedisPrefixEnum.TOKEN_MGR.toString() + adminUserBean.getToken(), adminUser, 20 * 60 * 60 * 1000);

            // 返回结果
            ExternalMapBean bean = new ExternalMapBean();
            bean.setToken(adminUserBean.getToken());
            bean.setUrl(frontTopUrl);
            bean.setUserid(adminUserBean.getAdminUserId());
            return bean;
        }
        return null;
    }
}
